/**
 * SECTION:tester
 * @short_description: module for gtk-doc unit test
 *
 * This file contains non-sense code for the sole purpose of testing the docs.
 *
 * As described in http://bugzilla.gnome.org/show_bug.cgi?id=457077 it
 * returns nothing.
 */

#include "tester.h"

/**
 * test:
 * @a: arg
 *
 * lonely function
 */
void test (gint a) {
}

/**
 * GtkDocTestIf:
 *
 * test object
 */
/**
 * GtkDocTestIfInterface:
 * @parent: parent interface type.
 *
 * test interface
 */

