version = "1.31"

# tools
dblatex = ''
fop = ''
pkg_config = '/usr/bin/pkg-config'
xsltproc = '/usr/bin/xsltproc'

# configured directories
prefix = '/usr/local'
datarootdir = "${prefix}/share".replace('${prefix}', prefix)
datadir = "${datarootdir}".replace('${datarootdir}', datarootdir)

exeext = ''
